@@ -1,4 +1,4 @@
-##' `Tidy up' R code and try to preserve comments.
+##' `Tidy up' R code while preserving comments.
 ##' This function has nothing to do with code optimization; it just
 ##' returns parsed source code, but also tries to preserve comments,
 ##' which is different with \code{\link[base]{parse}}. See `Details'.
@@ -14,34 +14,25 @@
 ##' \preformatted{  # asdf}
 ##'
 ##' It will be first masked as
-##' \preformatted{.SOME_IDENTIFIER = "  # asdf.ANOTHER_IDENTIFIER"}
+##' \preformatted{.SOME_IDENTIFIER <- "  # asdf.ANOTHER_IDENTIFIER"}
 ##'
 ##' which is a legal R expression, so \code{\link[base]{parse}} can
 ##' deal with it and will no longer remove the disguised comments.
 ##' In the end the identifiers will be removed to restore the original
-##' comments, i.e. the strings \code{'.SOME_IDENTIFIER = "'} and
+##' comments, i.e. the strings \code{'.SOME_IDENTIFIER <- "'} and
 ##' \code{'.ANOTHER_IDENTIFIER"'} are replaced with empty strings.
 ##'
-##' ``Inline'' comments are identified as ``two or more spaces'' plus
-##' the hash symbol \code{#} without any following double quotes in
-##' the line, e.g.
+##' ``Inline'' comments are handled differently: two spaces will be added
+##' before the hash symbol \code{#}, e.g.
+##' \preformatted{1+1#  comments}
+##'
+##' will become
 ##' \preformatted{1+1  #  comments}
 ##'
-##' or
-##' \preformatted{1+1    # comments}
-##'
-##' This might be dangerous to your source code, for instance,
-##' \preformatted{a = 'I am a string   #yes'}
-##'
-##' does not contain comments (\code{#} is inside a string), but this
-##' function will treat it as if it does! If you need to use the hash
-##' symbol in a string, you must put it in double quotes, e.g.
-##' \preformatted{a = "I am a string   #yes"}
-##'
 ##' Inline comments are first disguised as a weird operation with its
 ##' preceding R code, which is essentially meaningless but
 ##' syntactically correct!  For example,
-##' \preformatted{1+1 %InLiNe_IdEnTiFiEr% "   # comments"}
+##' \preformatted{1+1 %InLiNe_IdEnTiFiEr% "#  comments"}
 ##'
 ##' then \code{\link[base]{parse}} will deal with this expression;
 ##' again, the disguised comments will not be removed. In the end,
@@ -62,6 +53,8 @@
 ##' or not? (\code{FALSE} by default)
 ##' @param keep.space logical: whether to preserve the leading spaces
 ##' in the single lines of comments (default \code{FALSE})
+##' @param replace.assign logical: whether to replace the assign
+##' operator \code{=} with \code{<-}
 ##' @param output output to the console or a file using
 ##' \code{\link[base]{cat}}?
 ##' @param text an alternative way to specify the input: if it is
@@ -82,39 +75,46 @@
 ##' operator.}  \item{begin.comment,end.comment}{ identifiers used to
 ##' mark the comments }
 ##'
-##' @note When \code{keep.comment == TRUE}, \emph{you must not use
-##' double quotes in your inline comments -- only single quotes are
-##' allowed!!} For example, the code below will make the function
-##' discard the comments:
+##' @note When \code{keep.comment == TRUE}, \emph{all your double
+##' quotes in the comments will be replaced by single quotes!!} For
+##' example,
 ##' \preformatted{1 + 1  # here is the "comment"}
 ##'
-##' Instead, you have to write like this to protect the comments:
+##' will become
 ##' \preformatted{1 + 1  # here is the 'comment'}
 ##'
 ##' There are hidden options which can control the behaviour of this
 ##' function: the argument \code{keep.comment} gets its value from
 ##' \code{options('keep.comment')} by default;  \code{keep.blank.line}
-##' from \code{options('keep.blank.line')}, and \code{keep.space} from
-##' \code{options('keep.space')}. If these options are \code{NULL},
-##' the default values will be \code{TRUE}, \code{FALSE} and
-##' \code{FALSE} respectively.
+##' from \code{options('keep.blank.line')}, \code{keep.space} from
+##' \code{options('keep.space')}, and \code{replace.assign} from
+##' \code{options('replace.assign')}. If these options are
+##' \code{NULL}, the default values will be \code{TRUE}, \code{FALSE},
+##' \code{FALSE} and \code{FALSE} respectively.
 ##'
 ##' Also note that if \code{keep.space} is \code{FALSE}, single lines
 ##' of long comments will be wrapped into shorter ones automatically.
 ##' Otherwise, long comments will not be wrapped, so they may exceed
 ##' the page margin, and \code{\\\\t} will be replaced with
-##' \code{\\t}.
+##' \code{\\t}. Roxygen comments will not be wrapped in any case.
 ##'
 ##' \subsection{Warning}{ The best strategy to avoid failure is to put
 ##' comments in whole lines or after \emph{complete} R
 ##' expressions. Here are some examples which could make
 ##' \code{\link{tidy.source}} fail:
-##' \preformatted{if (TRUE) {  ## comments right after a curly brace } }
-##'
 ##' \preformatted{1 + 2 +   ## comments after an incomplete line
-##'
 ##'   3 + 4}
 ##'
+##' And the comments right after the curly brace will be moved to the
+##' next line, e.g.
+##' \preformatted{if (TRUE) {  ## comments right after a curly brace
+##' }}
+##'
+##' will become
+##' \preformatted{if (TRUE) {
+##'     ## comments right after a curly brace
+##' }}
+##'
 ##' }
 ##' @author Yihui Xie <\url{http://yihui.name}> with substantial
 ##' contribution from Yixuan Qiu <\url{http://yixuan.cos.name}>
@@ -130,9 +130,9 @@
 ##' ## use the 'text' argument
 ##' src = c("    # a single line of comments is preserved",
 ##' '1+1', '  ', 'if(TRUE){',
-##' "x=1  # comments begin with at least 2 spaces!", '}else{',
+##' "x=1  # inline comments", '}else{',
 ##' "x=2;print('Oh no... ask the right bracket to go away!')}",
-##' '1*3 # this comment will be dropped!',
+##' '1*3 # one space before this comment will become two!',
 ##' "2+2+2    # 'short comments'", "   ",
 ##' "lm(y~x1+x2)  ### only 'single quotes' are allowed in comments",
 ##' "1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1  ## comments after a long line",
@@ -187,7 +187,7 @@
 ##'
 ##'
 tidy.source = function(source = "clipboard", keep.comment,
-    keep.blank.line, keep.space, output = TRUE, text = NULL,
+    keep.blank.line, keep.space, replace.assign, output = TRUE, text = NULL,
     width.cutoff = 0.75 * getOption("width"), ...) {
     if (is.null(text)) {
         if (source == "clipboard" && Sys.info()["sysname"] == "Darwin") {
@@ -197,6 +197,10 @@ tidy.source = function(source = "clipboard", keep.comment,
     } else {
         text.lines = text
     }
+    if (identical(text.lines, '')) {
+        if (output) cat('\n', ...)
+        return('')
+    }
     if (missing(keep.comment)) {
         keep.comment = if (is.null(getOption('keep.comment'))) TRUE else getOption('keep.comment')
     }
@@ -206,6 +210,9 @@ tidy.source = function(source = "clipboard", keep.comment,
     if (missing(keep.space)) {
         keep.space = if (is.null(getOption('keep.space'))) FALSE else getOption('keep.space')
     }
+    if (missing(replace.assign)) {
+        replace.assign = if (is.null(getOption('replace.assign'))) FALSE else getOption('replace.assign')
+    }
     tidy.block = function(block.text) {
         exprs = base::parse(text = block.text)
         n = length(exprs)
@@ -229,25 +236,46 @@ tidy.source = function(source = "clipboard", keep.comment,
         }
         ## wrap long comments if you do not want to preserve leading spaces
         if (!keep.space) {
-            head.idx = which(head.comment)
+            ## don't wrap roxygen comments
+            head.idx = which(head.comment & !grepl("^[[:space:]]*#*#'", text.lines))
             k = 0 # k records how far the wrapped comments have pushed the index
             for (i in head.idx) {
                 j = i + k               # j records the real index
                 tmp = strwrap(text.lines[j], width = width.cutoff, prefix = "# ",
                               exdent = 2, initial = '')
-                text.lines[j] = tmp[1]
-                if (length(tmp) > 1) text.lines = append(text.lines, tmp[-1], j)
+                if (length(tmp) > 1) {
+                    text.lines[j] = tmp[1]
+                    text.lines = append(text.lines, tmp[-1], j)
+                }
                 k = k + length(tmp) - 1
             }
             head.comment = grepl('^[[:space:]]*#', text.lines)
         }
-        text.lines[head.comment] = sprintf("%s=\"%s%s\"",
+        text.lines[head.comment] = sprintf("%s<-\"%s%s\"",
                 begin.comment, text.lines[head.comment], end.comment)
         blank.line = grepl('^[[:space:]]*$', text.lines)
         if (any(blank.line) && isTRUE(keep.blank.line))
-            text.lines[blank.line] = sprintf("%s=\"%s\"", begin.comment, end.comment)
+            text.lines[blank.line] = sprintf("%s<-\"%s\"", begin.comment, end.comment)
         ## replace end-of-line comments to cheat R
-        text.lines[!head.comment] = sub("([ ]{2,}#[^\"]*)$", " %InLiNe_IdEnTiFiEr% \"\\1\"", text.lines[!head.comment])
+        enc = options(encoding = "native.enc")
+        out = attr(parser(text = text.lines), 'data')
+        options(enc)
+        out = subset(out, out$terminal)
+        if (nrow(out) > 0) {
+            if (replace.assign) {
+                out$text[out$token.desc=='EQ_ASSIGN'] = '<-'
+            }
+            ## is inline comment?
+            idx1 = c(FALSE, diff(out$line1)==0) & (out$token.desc=='COMMENT')
+            ## is last line '{'?
+            idx2 = c(FALSE, (out$text == '{')[-length(idx1)])
+            out$text[idx1] = gsub('"', "'", out$text[idx1])
+            idx = idx1 & (!idx2)
+            out$text[idx] = sprintf(' %%InLiNe_IdEnTiFiEr%% "%s"', out$text[idx])
+            idx = idx1 & idx2
+            out$text[idx] = sprintf('%s<-\"%s%s\"', begin.comment, out$text[idx], end.comment)
+            text.lines = tapply(out$text, out$line1, paste, collapse=' ')
+        }
         text.mask = tidy.block(text.lines)
         text.tidy = unmask.source(text.mask, replace.tab = keep.space)
     }
@@ -273,9 +301,8 @@ tidy.source = function(source = "clipboard", keep.comment,
 ##'
 ##' src = c("    # a single line of comments is preserved",
 ##' '1+1', '  ', 'if(TRUE){',
-##' "x=1  # comments begin with at least 2 spaces!", '}else{',
+##' "x=1  # inline comments!", '}else{',
 ##' "x=2;print('Oh no... ask the right bracket to go away!')}",
-##' '1*3 # this comment will be dropped!',
 ##' "2+2+2    # 'short comments'",
 ##' "lm(y~x1+x2)  ### only 'single quotes' are allowed in comments",
 ##' "1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1  ## comments after a long line")
@@ -287,192 +314,16 @@ tidy.source = function(source = "clipboard", keep.comment,
 ##' cat(unmask.source(x), sep = '\n')
 ##'
 unmask.source = function(text.mask, replace.tab = FALSE) {
-    idx = grepl('\\.BeGiN_TiDy_IdEnTiFiEr_HaHaHa = ', text.mask)
+    idx = grepl('\\.BeGiN_TiDy_IdEnTiFiEr_HaHaHa <- ', text.mask)
     text.mask[idx] = gsub('\\\\', '\\', text.mask[idx], fixed = TRUE)
-    text.tidy = gsub("\\.BeGiN_TiDy_IdEnTiFiEr_HaHaHa = \"|\\.HaHaHa_EnD_TiDy_IdEnTiFiEr\"", "", text.mask)
+    text.tidy = gsub("\\.BeGiN_TiDy_IdEnTiFiEr_HaHaHa <- \"|\\.HaHaHa_EnD_TiDy_IdEnTiFiEr\"", "", text.mask)
     ## if the comments were separated into the next line, then remove '\n' after
     ##   the identifier first to move the comments back to the same line
-    text.tidy = gsub(" %InLiNe_IdEnTiFiEr%[ ]*[^\n]*\"([ ]{2,}#[^\"]*)\"", "\\1",
-    gsub("%InLiNe_IdEnTiFiEr%[ ]*\n", "%InLiNe_IdEnTiFiEr%", text.tidy))
+    text.tidy = gsub(" %InLiNe_IdEnTiFiEr%[ ]*[^\n]*\"([ ]*#[^\"]*)\"", "  \\1",
+                     gsub("%InLiNe_IdEnTiFiEr%[ ]*\n", "%InLiNe_IdEnTiFiEr%", text.tidy))
     if (replace.tab) gsub('\\\\t', '\t', text.tidy) else text.tidy
 }
 
-##' A GUI to format R code.
-##' Create a GUI (via GTK+ by default) to format R code.
-##'
-##' This function calls \code{\link{tidy.source}} to format R code.
-##' Spaces and indent will be added to the code automatically.
-##'
-##' We can either open an R source file or directly write R code in the text
-##' widget. Click the ``convert'' button, and the code will become tidy. See
-##' \code{\link{tidy.source}} for more details.
-##'
-##' @param guiToolkit the GUI toolkit to use
-##' @return the text widget is returned
-##' @note
-##' By default, the interface is based on GTK+ (R package \bold{RGtk2}), but
-##' other options (\bold{tcltk}, \bold{rJava} and \bold{Qt}) are possible too. See the
-##' examples below. Note the ``Font'' button is only for the GTK+ interface.
-##' @author Yihui Xie <\url{http://yihui.name}>
-##' @seealso \code{\link{tidy.source}}
-##' @export
-##' @references
-##'   \url{http://yihui.name/en/2010/04/formatr-farewell-to-ugly-r-code/}
-##' @examples
-##'
-##' if (interactive() && require('gWidgetsRGtk2')) {
-##'
-##' ## a GUI will show up on loading if one of the gWidgets
-##' ##   toolkit is present (e.g. via library(gWidgetsRGtk2))
-##' library(formatR)
-##'
-##' g = formatR()
-##'
-##' ## we have control over the text widget, e.g. set or get the text
-##'
-##' svalue(g) = c("# a single line of comments is preserved", "1+1",
-##'     "if(TRUE){", paste("x=1 ",
-##'     "# comments begin with at least 2 spaces!"), "}else{",
-##'     "x=2;print('Oh no... ask the right bracket to go away!')}",
-##'     "1*3 # this comment will be dropped!")
-##'
-##' ## click 'Convert' now, and see
-##'
-##' cat(svalue(g), sep = '\n')   # get its value
-##'
-##' ## tcl/tk interface: need gWidgetstcltk package
-##' formatR('tcltk')
-##'
-##' ## Java interface: need gWidgetsrJava package
-##' formatR('rJava')
-##'
-##' }
-##'
-formatR = function(guiToolkit = 'RGtk2') {
-    options(guiToolkit = guiToolkit)
-    stopifnot(require(paste('gWidgets', guiToolkit, sep = ''), character.only = TRUE))
-
-    g = ggroup(horizontal = FALSE, container = gwindow("Tidy R Source"))
-    g1 = ggroup(container = g, expand = TRUE)
-    g2 = ggroup(container = g)
-
-    txt = gtext(container = g1, wrap = FALSE, font.attr = c(family = "monospace",
-        size = "medium"), expand = TRUE)
-    tag(txt, "font.attr") = c(family = "monospace", size = "medium",
-        weight = "light")
-    tag(txt, "tidy.opt") = list(keep.comment = TRUE, keep.blank.line = TRUE,
-        width.cutoff = 60)
-    gbutton("Open", container = g2, handler = function(h, ...) {
-        s = gfile("Open R Source")
-        if (!is.na(s)) {
-            svalue(txt) = readLines(s)
-            tag(txt, "src.file") = s
-            tooltip(txt) = s
-        }
-    })
-    gbutton("Convert", container = g2, handler = function(h,
-        ...) {
-        tidy.opt = tag(txt, "tidy.opt")
-        src = svalue(txt)
-        Encoding(src) = 'UTF-8'
-        src = unlist(strsplit(src, '\n', fixed = TRUE))
-        text.tidy = tidy.source(text = src,
-            keep.comment = tidy.opt$keep.comment,
-            keep.blank.line = tidy.opt$keep.blank.line,
-            width.cutoff = tidy.opt$width.cutoff,
-            output = FALSE)$text.tidy
-        svalue(txt) = text.tidy
-    })
-    gbutton("Save", container = g2, handler = function(h, ...) {
-        s = tag(txt, "src.file")
-        if (is.null(s))
-            s = gfile("Save", type = "save")
-        if (!is.na(s))
-            writeLines(svalue(txt), s)
-    })
-    gbutton("Save-as", container = g2, handler = function(h,
-        ...) {
-        s = gfile("Save as", type = "save")
-        if (!is.na(s))
-            writeLines(svalue(txt), s)
-    })
-    gbutton("Execute", container = g2, handler = function(h,
-        ...) {
-        src = svalue(txt)
-        Encoding(src) = 'UTF-8'
-        zz = textConnection(src)
-        source(zz, max.deparse.length = Inf, echo = TRUE)
-        close(zz)
-    })
-    if (getOption("guiToolkit") == "RGtk2") {
-        gbutton("Select-font", container = g2, handler = function(h,
-            ...) {
-            w = gwindow("Font Specification", )
-            g = ggroup(horizontal = FALSE, container = w)
-            tbl = glayout(container = g, expand = TRUE, spacing = 0)
-            ft = tag(txt, "font.attr")
-            tbl[1, 1, expand = TRUE] = (gf.size <- gframe("Size",
-                container = tbl))
-            r.size = gradio(tag(txt)$tags$sizes, which(ft["size"] ==
-                tag(txt)$tags$sizes), horizontal = TRUE, container = gf.size)
-            tbl[2, 1, expand = TRUE] = (gf.weight <- gframe("Weight",
-                container = tbl))
-            r.weight = gradio(tag(txt)$tags$weight, which(ft["weight"] ==
-                tag(txt)$tags$weight), horizontal = TRUE, container = gf.weight)
-            g1 = ggroup(container = g)
-            b.ok = gbutton("OK", container = g1, handler = function(h,
-                ...) {
-                ft = c(family = "monospace", size = svalue(r.size),
-                  weight = svalue(r.weight))
-                font(txt) = ft
-                tag(txt, "font.attr") = ft
-                dispose(w)
-            })
-            b.cancel = gbutton("Cancel", container = g1, handler = function(h,
-                ...) {
-                dispose(w)
-            })
-        })
-    }
-    gbutton("Preferences", container = g2, handler = function(h,
-        ...) {
-        w = gwindow("Preferences")
-        g = ggroup(horizontal = FALSE, container = w)
-        tbl = glayout(container = g, expand = TRUE, spacing = 0)
-        tidy.opt = tag(txt, "tidy.opt")
-        tbl[1, 1, expand = TRUE] = (gf.kc <- gframe("Keep Comments?",
-            container = tbl))
-        r.kc = gradio(c("TRUE", "FALSE"), which(as.logical(tidy.opt$keep.comment) ==
-            c(TRUE, FALSE)), horizontal = TRUE, container = gf.kc)
-        tbl[2, 1, expand = TRUE] = (gf.kb <- gframe("Keep Blank Lines?",
-            container = tbl))
-        r.kb = gradio(c("TRUE", "FALSE"), which(as.logical(tidy.opt$keep.blank.line) ==
-            c(TRUE, FALSE)), horizontal = TRUE, container = gf.kb)
-        tbl[3, 1, expand = TRUE] = (gf.wi <- gframe("Text Width",
-            container = tbl))
-        r.wi = gedit(as.character(tidy.opt$width.cutoff), container = gf.wi,
-            coerce.with = as.integer)
-        g1 = ggroup(container = g)
-        b.ok = gbutton("OK", container = g1, handler = function(h,
-            ...) {
-            if (is.na(svalue(r.wi)))
-                gmessage("Please input an integer for the text width!",
-                  "Error", icon = "error")
-            else {
-                tag(txt, "tidy.opt") = list(keep.comment = as.logical(svalue(r.kc)),
-                  keep.blank.line = as.logical(svalue(r.kb)),
-                  width.cutoff = svalue(r.wi))
-                dispose(w)
-            }
-        })
-        b.cancel = gbutton("Cancel", container = g1, handler = function(h,
-            ...) {
-            dispose(w)
-        })
-    })
-    invisible(txt)
-}
-
 
 ##' A weird operator for internal use only.
 ##' This operator is almost meaningless; it is used to mask the inline comments.
